@@ -22,15 +22,15 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from __future__ import print_function
 from os import stat
 from os.path import exists, getsize
 import sys
 import glob
-import string
 import gzip
 from optparse import OptionParser
 
-__version__ = '0.5.1'
+__version__ = '0.6.1'
 
 
 PY3 = sys.version_info[0] == 3
@@ -54,16 +54,23 @@ class Pygtail(object):
     Keyword arguments:
     offset_file   File to which offset data is written (default: <logfile>.offset).
     paranoid      Update the offset file every time we read a line (as opposed to
-                  only when we reach the end of the file (default: False)
+                  only when we reach the end of the file (default: False))
+    every_n       Update the offset file every n'th line (as opposed to only when
+                  we reach the end of the file (default: 0))
+    on_update     Execute this function when offset data is written (default False)
     copytruncate  Support copytruncate-style log rotation (default: True)
     """
-    def __init__(self, filename, offset_file=None, paranoid=False, copytruncate=True):
+    def __init__(self, filename, offset_file=None, paranoid=False, copytruncate=True,
+                 every_n=0, on_update=False):
         self.filename = filename
         self.paranoid = paranoid
+        self.every_n = every_n
+        self.on_update = on_update
         self.copytruncate = copytruncate
         self._offset_file = offset_file or "%s.offset" % self.filename
         self._offset_file_inode = 0
         self._offset = 0
+        self._since_update = 0
         self._fh = None
         self._rotated_logfile = None
 
@@ -92,7 +99,7 @@ class Pygtail(object):
         Return the next line in the file, updating the offset.
         """
         try:
-            line = next(self._filehandle())
+            line = self._get_next_line()
         except StopIteration:
             # we've reached the end of the file; if we're processing the
             # rotated log file, we can continue with the actual file; otherwise
@@ -103,7 +110,7 @@ class Pygtail(object):
                 self._offset = 0
                 # open up current logfile and continue
                 try:
-                    line = next(self._filehandle())
+                    line = self._get_next_line()
                 except StopIteration:  # oops, empty file
                     self._update_offset_file()
                     raise
@@ -113,6 +120,8 @@ class Pygtail(object):
 
         if self.paranoid:
             self._update_offset_file()
+        elif self.every_n and self.every_n <= self._since_update:
+            self._update_offset_file()
 
         return line
 
@@ -161,7 +170,7 @@ class Pygtail(object):
             if filename.endswith('.gz'):
                 self._fh = gzip.open(filename, 'r')
             else:
-                self._fh = open(filename, "r")
+                self._fh = open(filename, "r", 1)
             self._fh.seek(self._offset)
 
         return self._fh
@@ -170,11 +179,14 @@ class Pygtail(object):
         """
         Update the offset file with the current inode and offset.
         """
+        if self.on_update:
+            self.on_update()
         offset = self._filehandle().tell()
         inode = stat(self.filename).st_ino
         fh = open(self._offset_file, "w")
         fh.write("%s\n%s\n" % (inode, offset))
         fh.close()
+        self._since_update = 0
 
     def _determine_rotated_logfile(self):
         """
@@ -236,6 +248,13 @@ class Pygtail(object):
         # no match
         return None
 
+    def _get_next_line(self):
+        line = self._filehandle().readline()
+        if not line:
+            raise StopIteration
+        self._since_update += 1
+        return line
+
 
 def main():
     # command-line parsing
@@ -246,6 +265,9 @@ def main():
     cmdline.add_option("--paranoid", "-p", action="store_true",
         help="Update the offset file every time we read a line (as opposed to"
              " only when we reach the end of the file).")
+    cmdline.add_option("--every-n", "-n", action="store",
+        help="Update the offset file every n'th time we read a line (as opposed to"
+             " only when we reach the end of the file).")
     cmdline.add_option("--no-copytruncate", action="store_true",
         help="Don't support copytruncate-style log rotation. Instead, if the log file"
              " shrinks, print a warning.")
@@ -261,9 +283,12 @@ def main():
     if (len(args) != 1):
         cmdline.error("Please provide a logfile to read.")
 
+    if options.every_n:
+        options.every_n = int(options.every_n)
     pygtail = Pygtail(args[0],
                       offset_file=options.offset_file,
                       paranoid=options.paranoid,
+                      every_n=options.every_n,
                       copytruncate=not options.no_copytruncate)
 
     for line in pygtail:
